@@ -28,18 +28,20 @@ graph.maxflow <- function(graph, source, target, capacity=NULL) {
   if (is.null(capacity)) {
     if ("capacity" %in% list.edge.attributes(graph)) {
       capacity <- E(graph)$capacity
-    } else {
-      stop("capacity argument is not given and no `capacity' attribute")
     }
   }
-  capacity <- as.numeric(capacity)
+  if (!is.null(capacity)) {
+    capacity <- as.numeric(capacity)
+  }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_maxflow", graph, as.numeric(source), as.numeric(target),
         capacity,
         PACKAGE="igraph")
 }
 
-graph.mincut <- function(graph, source=NULL, target=NULL, capacity=NULL) {
+graph.mincut <- function(graph, source=NULL, target=NULL, capacity=NULL,
+                         value.only=TRUE) {
 
   if (!is.igraph(graph)) {
     stop("Not a graph object")
@@ -47,19 +49,26 @@ graph.mincut <- function(graph, source=NULL, target=NULL, capacity=NULL) {
   if (is.null(capacity)) {
     if ("capacity" %in% list.edge.attributes(graph)) {
       capacity <- E(graph)$capacity
-    } else {
-      stop("capacity argument is not given and no `capacity' attribute")
     }
   }
   if (is.null(source) && !is.null(target) ||
       is.null(target) && !is.null(source)) {
     stop("Please give both source and target oe neither")
   }
-  capacity <- as.numeric(capacity)
-  
+  if (!is.null(capacity)) {
+    capacity <- as.numeric(capacity)
+  }
+
+  value.only <- as.logical(value.only)
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   if (is.null(target) && is.null(source)) {
-    .Call("R_igraph_mincut_value", graph, capacity,
-          PACKAGE="igraph")
+    if (value.only) {
+      .Call("R_igraph_mincut_value", graph, capacity,
+            PACKAGE="igraph")
+    } else {
+      .Call("R_igraph_mincut", graph, capacity,
+            PACKAGE="igraph")
+    }
   } else {
     .Call("R_igraph_st_mincut_value", graph, as.numeric(source),
           as.numeric(target), capacity,
@@ -74,9 +83,11 @@ vertex.connectivity <- function(graph, source=NULL, target=NULL, checks=TRUE) {
   }
 
   if (is.null(source) && is.null(target)) {
+    on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
     .Call("R_igraph_vertex_connectivity", graph, as.logical(checks),
           PACKAGE="igraph")
   } else if (!is.null(source) && !is.null(target)) {
+    on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
     .Call("R_igraph_st_vertex_connectivity", graph, as.numeric(source),
           as.numeric(target),
           PACKAGE="igraph")
@@ -92,9 +103,11 @@ edge.connectivity <- function(graph, source=NULL, target=NULL, checks=TRUE) {
   }
 
   if (is.null(source) && is.null(target)) {    
+    on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
     .Call("R_igraph_edge_connectivity", graph, as.logical(checks),
           PACKAGE="igraph")
   } else if (!is.null(source) && !is.null(target)) {
+    on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
     .Call("R_igraph_st_edge_connectivity", graph,
           as.numeric(source), as.numeric(target),
           PACKAGE="igraph")
@@ -109,6 +122,7 @@ edge.disjoint.paths <- function(graph, source, target) {
     stop("Not a graph object")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_edge_disjoint_paths", graph,
         as.numeric(source), as.numeric(target),
         PACKAGE="igraph")
@@ -120,6 +134,7 @@ vertex.disjoint.paths <- function(graph, source=NULL, target=NULL) {
     stop("Not a graph object")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_vertex_disjoint_paths", graph, as.numeric(source),
         as.numeric(target),
         PACKAGE="igraph")
@@ -131,6 +146,7 @@ graph.adhesion <- function(graph, checks=TRUE) {
     stop("Not a graph object")
   }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_adhesion", graph, as.logical(checks),
         PACKAGE="igraph")
 }
@@ -141,6 +157,7 @@ graph.cohesion <- function(graph, checks=TRUE) {
     stop("Not a graph object")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_cohesion", graph, as.logical(checks),
         PACKAGE="igraph")
 }
