@@ -33,250 +33,6 @@
 #define PARENT(x)     (((x)+1)/2-1)
 #define LEFTCHILD(x)  (((x)+1)*2-1)
 #define RIGHTCHILD(x) (((x)+1)*2)
-  
-/**
- * \ingroup heap
- * \brief Initializes an empty heap object (constructor).
- *
- * @return Error code:
- *         - <b>IGRAPH_ENOMEM</b>: out of memory
- */
-
-int igraph_heap_init           (igraph_heap_t* h, long int alloc_size) {
-  if (alloc_size <= 0 ) { alloc_size=1; }
-  h->stor_begin=Calloc(alloc_size, igraph_real_t);
-  if (h->stor_begin==0) {
-    IGRAPH_ERROR("heap init failed", IGRAPH_ENOMEM);
-  }
-  h->stor_end=h->stor_begin + alloc_size;
-  h->end=h->stor_begin;
-  h->destroy=1;
-  
-  return 0;  
-}
-
-/**
- * \ingroup heap
- * \brief Initializes a heap object from an array, the heap is also
- * built of course (constructor).
- *
- * @return Error code:
- *         - <b>IGRAPH_ENOMEM</b>: out of memory
- */
-
-int igraph_heap_init_array     (igraph_heap_t *h, igraph_real_t* data, long int len) {
-  h->stor_begin=Calloc(len, igraph_real_t);
-  if (h->stor_begin==0) {
-    IGRAPH_ERROR("heap init from array failed", IGRAPH_ENOMEM);
-  }
-  h->stor_end=h->stor_begin+len;
-  h->end=h->stor_end;
-  h->destroy=1;
-
-  memcpy(h->stor_begin, data, len*sizeof(igraph_real_t));
-
-  igraph_heap_i_build (h->stor_begin, h->end-h->stor_begin, 0);
-  
-  return 0;
-}
-
-/**
- * \ingroup heap
- * \brief Destroys an initialized heap object.
- */
-
-void igraph_heap_destroy        (igraph_heap_t* h) {  
-  if (h->destroy) {
-    if (h->stor_begin != 0) {
-      Free(h->stor_begin);
-      h->stor_begin=0;
-    }
-  }
-}
-
-/**
- * \ingroup heap
- * \brief Decides whether a heap object is empty.
- */
-
-igraph_bool_t igraph_heap_empty          (igraph_heap_t* h) {
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-  return h->stor_begin == h->end;
-}
-
-/**
- * \ingroup heap
- * \brief Adds an element to a heap object.
- */
-
-int igraph_heap_push           (igraph_heap_t* h, igraph_real_t elem) {
-  int ret;
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-	
-  /* full, allocate more storage */
-  if (h->stor_end == h->end) {
-    long int new_size = igraph_heap_size(h) * 2;
-    if (new_size == 0) { new_size = 1; }
-    IGRAPH_CHECK(ret=igraph_heap_reserve(h, new_size));
-  }
-	
-  *(h->end) = elem;
-  h->end += 1;
-
-  /* maintain heap */
-  igraph_heap_i_shift_up(h->stor_begin, igraph_heap_size(h), igraph_heap_size(h)-1);
-	
-  return 0;
-}
-
-/**
- * \ingroup heap
- * \brief Returns the largest element in a heap.
- */
-
-igraph_real_t igraph_heap_max       (igraph_heap_t* h) {
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-  assert(h->stor_begin != h->end);
-  
-  return h->stor_begin[0];
-}
-
-/**
- * \ingroup heap
- * \brief Returns and removes the largest element in a heap.
- */
-
-igraph_real_t igraph_heap_delete_max(igraph_heap_t* h) {
-  igraph_real_t tmp;
-
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-
-  tmp=h->stor_begin[0];
-  igraph_heap_i_switch(h->stor_begin, 0, igraph_heap_size(h)-1);
-  h->end -= 1;
-  igraph_heap_i_sink(h->stor_begin, h->end-h->stor_begin, 0);
-  
-  return tmp;
-}
-
-/**
- * \ingroup heap
- * \brief Gives the number of elements in a heap.
- */
-
-long int igraph_heap_size      (igraph_heap_t* h) {
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-  return h->end - h->stor_begin;
-}
-
-/**
- * \ingroup heap
- * \brief Allocates more memory for a heap if needed.
- *
- * @return Error code:
- *         - <b>IGRAPH_ENOMEM</b>: out of memory
- */
-
-int igraph_heap_reserve        (igraph_heap_t* h, long int size) {
-  long int actual_size=igraph_heap_size(h);
-  igraph_real_t *tmp;
-  assert(h != NULL);
-  assert(h->stor_begin != NULL);
-  
-  if (size <= actual_size) { return 0; }
-  
-  tmp=Realloc(h->stor_begin, size, igraph_real_t);
-  if (tmp==0) {
-    IGRAPH_ERROR("heap reserve failed", IGRAPH_ENOMEM);
-  }
-  h->stor_begin=tmp;
-  h->stor_end=h->stor_begin + size;
-  h->end=h->stor_begin+actual_size;
-  
-  return 0;
-}
-
-/**
- * \ingroup heap
- * \brief Build a heap, this should not be called directly.
- */
-
-void igraph_heap_i_build(igraph_real_t* arr, long int size, long int head) {
-
-  if (RIGHTCHILD(head) < size) { 
-    /* both subtrees */
-    igraph_heap_i_build(arr, size, LEFTCHILD(head) );
-    igraph_heap_i_build(arr, size, RIGHTCHILD(head));
-    igraph_heap_i_sink(arr, size, head);
-  } else if (LEFTCHILD(head) < size) {
-    /* only left */
-    igraph_heap_i_build(arr, size, LEFTCHILD(head));
-    igraph_heap_i_sink(arr, size, head);
-  } else {
-    /* none */
-  }
-}
-
-/**
- * \ingroup heap
- * \brief Shift an element upwards in a heap, this should not be
- * called directly.
- */
-
-void igraph_heap_i_shift_up(igraph_real_t* arr, long int size, long int elem) {
-  
-  if (elem==0 || arr[elem] < arr[PARENT(elem)]) { 
-    /* at the top */
-  } else {
-    igraph_heap_i_switch(arr, elem, PARENT(elem));
-    igraph_heap_i_shift_up(arr, size, PARENT(elem));
-  }
-}
-
-/**
- * \ingroup heap
- * \brief Moves an element down in a heap, this function should not be
- * called directly.
- */
-
-void igraph_heap_i_sink(igraph_real_t* arr, long int size, long int head) {
-
-  if (LEFTCHILD(head) >= size) { 
-    /* no subtrees */
-  } else if (RIGHTCHILD(head) == size ||
-	     arr[LEFTCHILD(head)] >= arr[RIGHTCHILD(head)]) {
-    /* sink to the left if needed */
-    if (arr[head] < arr[LEFTCHILD(head)]) {
-      igraph_heap_i_switch(arr, head, LEFTCHILD(head));
-      igraph_heap_i_sink(arr, size, LEFTCHILD(head));
-    }
-  } else {
-    /* sink to the right */
-    if (arr[head] < arr[RIGHTCHILD(head)]) {
-      igraph_heap_i_switch(arr, head, RIGHTCHILD(head));
-      igraph_heap_i_sink(arr, size, RIGHTCHILD(head));
-    }
-  }
-}
-
-/**
- * \ingroup heap
- * \brief Switches two elements in a heap, this function should not be
- * called directly.
- */
-
-void igraph_heap_i_switch(igraph_real_t* arr, long int e1, long int e2) {
-  if (e1!=e2) {
-    igraph_real_t tmp=arr[e1];
-    arr[e1]=arr[e2];
-    arr[e2]=tmp;
-  }
-}
 
 /**
  * \ingroup indheap
@@ -288,14 +44,14 @@ void igraph_heap_i_switch(igraph_real_t* arr, long int e1, long int e2) {
 
 int igraph_indheap_init           (igraph_indheap_t* h, long int alloc_size) {
  if (alloc_size <= 0 ) { alloc_size=1; }
- h->stor_begin=Calloc(alloc_size, igraph_real_t);
+ h->stor_begin=igraph_Calloc(alloc_size, igraph_real_t);
  if (h->stor_begin==0) {
    h->index_begin=0;
    IGRAPH_ERROR("indheap init failed", IGRAPH_ENOMEM);
  }
- h->index_begin=Calloc(alloc_size, long int);
+ h->index_begin=igraph_Calloc(alloc_size, long int);
  if (h->index_begin==0) {
-   Free(h->stor_begin);
+   igraph_Free(h->stor_begin);
    h->stor_begin=0;
    IGRAPH_ERROR("indheap init failed", IGRAPH_ENOMEM);
  }
@@ -318,14 +74,14 @@ int igraph_indheap_init           (igraph_indheap_t* h, long int alloc_size) {
 int igraph_indheap_init_array     (igraph_indheap_t *h, igraph_real_t* data, long int len) {
   long int i;
 
-  h->stor_begin=Calloc(len, igraph_real_t);
+  h->stor_begin=igraph_Calloc(len, igraph_real_t);
   if (h->stor_begin==0) {
     h->index_begin=0;
     IGRAPH_ERROR("indheap init from array failed", IGRAPH_ENOMEM);
   }
-  h->index_begin=Calloc(len, long int);
+  h->index_begin=igraph_Calloc(len, long int);
   if (h->index_begin==0) {
-    Free(h->stor_begin);
+    igraph_Free(h->stor_begin);
     h->stor_begin=0;
     IGRAPH_ERROR("indheap init from array failed", IGRAPH_ENOMEM);
   }
@@ -352,11 +108,11 @@ void igraph_indheap_destroy        (igraph_indheap_t* h) {
   assert(h != 0);
   if (h->destroy) {
     if (h->stor_begin != 0) {
-      Free(h->stor_begin);
+      igraph_Free(h->stor_begin);
       h->stor_begin=0;
     }
     if (h->index_begin != 0) {
-      Free(h->index_begin);
+      igraph_Free(h->index_begin);
       h->index_begin=0;
     }
   }
@@ -511,20 +267,20 @@ int igraph_indheap_reserve        (igraph_indheap_t* h, long int size) {
   
   if (size <= actual_size) { return 0; }
 
-  tmp1=Calloc(size, igraph_real_t);  
+  tmp1=igraph_Calloc(size, igraph_real_t);  
   if (tmp1==0) {
     IGRAPH_ERROR("indheap reserve failed", IGRAPH_ENOMEM);
   }
   IGRAPH_FINALLY(free, tmp1); 	/* TODO: hack */
-  tmp2=Calloc(size, long int);
+  tmp2=igraph_Calloc(size, long int);
   if (tmp2==0) {
     IGRAPH_ERROR("indheap reserve failed", IGRAPH_ENOMEM);
   }
   IGRAPH_FINALLY(free, tmp2);
   memcpy(tmp1, h->stor_begin, actual_size*sizeof(igraph_real_t));
   memcpy(tmp2, h->index_begin, actual_size*sizeof(long int));
-  Free(h->stor_begin);
-  Free(h->index_begin);
+  igraph_Free(h->stor_begin);
+  igraph_Free(h->index_begin);
   
   h->stor_begin=tmp1;
   h->index_begin=tmp2;
@@ -641,7 +397,7 @@ void igraph_indheap_i_switch(igraph_indheap_t* h, long int e1, long int e2) {
 
 int igraph_d_indheap_init           (igraph_d_indheap_t* h, long int alloc_size) {
  if (alloc_size <= 0 ) { alloc_size=1; }
-  h->stor_begin=Calloc(alloc_size, igraph_real_t);
+  h->stor_begin=igraph_Calloc(alloc_size, igraph_real_t);
   if (h->stor_begin==0) {
     h->index_begin=0;
     h->index2_begin=0;
@@ -650,17 +406,17 @@ int igraph_d_indheap_init           (igraph_d_indheap_t* h, long int alloc_size)
   h->stor_end=h->stor_begin + alloc_size;
   h->end=h->stor_begin;
   h->destroy=1;
-  h->index_begin=Calloc(alloc_size, long int);
+  h->index_begin=igraph_Calloc(alloc_size, long int);
   if (h->index_begin==0) {
-    Free(h->stor_begin);
+    igraph_Free(h->stor_begin);
     h->stor_begin=0;
     h->index2_begin=0;
     IGRAPH_ERROR("d_indheap init failed", IGRAPH_ENOMEM);
   }
-  h->index2_begin=Calloc(alloc_size, long int);
+  h->index2_begin=igraph_Calloc(alloc_size, long int);
   if (h->index2_begin==0) {
-    Free(h->stor_begin);
-    Free(h->index_begin);
+    igraph_Free(h->stor_begin);
+    igraph_Free(h->index_begin);
     h->stor_begin=0;
     h->index_begin=0;
     IGRAPH_ERROR("d_indheap init failed", IGRAPH_ENOMEM);
@@ -678,15 +434,15 @@ void igraph_d_indheap_destroy        (igraph_d_indheap_t* h) {
   assert(h != 0);
   if (h->destroy) {
     if (h->stor_begin != 0) {
-      Free(h->stor_begin);
+      igraph_Free(h->stor_begin);
       h->stor_begin=0;
     }
     if (h->index_begin != 0) {
-      Free(h->index_begin);
+      igraph_Free(h->index_begin);
       h->index_begin=0;
     }
     if (h->index2_begin != 0) {
-      Free(h->index2_begin);
+      igraph_Free(h->index2_begin);
       h->index2_begin=0;
     }
   }
@@ -791,17 +547,17 @@ int igraph_d_indheap_reserve        (igraph_d_indheap_t* h, long int size) {
   
   if (size <= actual_size) { return 0; }
 
-  tmp1=Calloc(size, igraph_real_t);
+  tmp1=igraph_Calloc(size, igraph_real_t);
   if (tmp1==0) {
     IGRAPH_ERROR("d_indheap reserve failed", IGRAPH_ENOMEM);
   }
   IGRAPH_FINALLY(free, tmp1);	/* TODO: hack */
-  tmp2=Calloc(size, long int);
+  tmp2=igraph_Calloc(size, long int);
   if (tmp2==0) {
     IGRAPH_ERROR("d_indheap reserve failed", IGRAPH_ENOMEM);
   }
   IGRAPH_FINALLY(free, tmp2);	/* TODO: hack */
-  tmp3=Calloc(size, long int);
+  tmp3=igraph_Calloc(size, long int);
   if (tmp3==0) {
     IGRAPH_ERROR("d_indheap reserve failed", IGRAPH_ENOMEM);
   }
@@ -810,9 +566,9 @@ int igraph_d_indheap_reserve        (igraph_d_indheap_t* h, long int size) {
   memcpy(tmp1, h->stor_begin, actual_size*sizeof(igraph_real_t));
   memcpy(tmp2, h->index_begin, actual_size*sizeof(long int));
   memcpy(tmp3, h->index2_begin, actual_size*sizeof(long int));
-  Free(h->stor_begin);
-  Free(h->index_begin);
-  Free(h->index2_begin);
+  igraph_Free(h->stor_begin);
+  igraph_Free(h->index_begin);
+  igraph_Free(h->index2_begin);
 
   h->stor_begin=tmp1;
   h->stor_end=h->stor_begin + size;
