@@ -115,8 +115,7 @@ int igraph_layout_random_3d(const igraph_t *graph, igraph_matrix_t *res) {
 /**
  * \ingroup layout
  * \function igraph_layout_circle
- * \brief Places the vertices uniformly on a circle, in the order of
- * vertex ids.
+ * \brief Places the vertices uniformly on a circle, in the order of vertex ids.
  * 
  * \param graph Pointer to an initialized graph object.
  * \param res Pointer to an initialized graph object. This will
@@ -204,8 +203,7 @@ int igraph_layout_sphere(const igraph_t *graph, igraph_matrix_t *res) {
 /**
  * \ingroup layout
  * \function igraph_layout_fruchterman_reingold
- * \brief Places the vertices on a plane according to the
- * Fruchterman-Reingold algorithm.
+ * \brief Places the vertices on a plane according to the Fruchterman-Reingold algorithm.
  *
  * </para><para>
  * This is a force-directed layout, see Fruchterman, T.M.J. and
@@ -227,6 +225,9 @@ int igraph_layout_sphere(const igraph_t *graph, igraph_matrix_t *res) {
  * \param use_seed Logical, if true the supplied values in the
  *        \p res argument are used as an initial layout, if
  *        false a random initial layout is used.
+ * \param weight Pointer to a vector containing edge weights, 
+ *        the attraction along the edges will be multiplied by these. 
+ *        It will be ignored if it is a null-pointer.
  * \return Error code.
  * 
  * Time complexity: O(|V|^2) in each
@@ -237,7 +238,8 @@ int igraph_layout_sphere(const igraph_t *graph, igraph_matrix_t *res) {
 int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *res,
 				       igraph_integer_t niter, igraph_real_t maxdelta,
 				       igraph_real_t area, igraph_real_t coolexp, 
-				       igraph_real_t repulserad, igraph_bool_t use_seed) {
+				       igraph_real_t repulserad, igraph_bool_t use_seed,
+				       const igraph_vector_t *weight) {
   igraph_real_t frk,t,ded,xd,yd;
   igraph_real_t rf,af;
   long int i,j,k;
@@ -247,6 +249,10 @@ int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *r
   igraph_eit_t edgeit;
   igraph_integer_t from, to;
   
+  if (weight && igraph_vector_size(weight) != igraph_ecount(graph)) {
+    IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+  }
+  
   IGRAPH_CHECK(igraph_matrix_resize(res, no_of_nodes, 2));
   if (!use_seed) {
     IGRAPH_CHECK(igraph_layout_random(graph, res));
@@ -289,7 +295,9 @@ int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *r
     /* Calculate the attractive "force" */
     IGRAPH_EIT_RESET(edgeit);
     while (!IGRAPH_EIT_END(edgeit)) {
-      igraph_edge(graph, IGRAPH_EIT_GET(edgeit), &from, &to);
+      long int edge=IGRAPH_EIT_GET(edgeit);
+      igraph_real_t w= weight ? VECTOR(*weight)[ edge ] : 1.0;
+      igraph_edge(graph, edge, &from, &to);
       j=from; 
       k=to;
       xd=MATRIX(*res, j, 0)-MATRIX(*res, k, 0);
@@ -299,7 +307,7 @@ int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *r
 	xd/=ded;                /* Rescale differences to length 1 */
 	yd/=ded;
       }
-      af=ded*ded/frk;
+      af=ded*ded/frk*w;
       MATRIX(dxdy, j, 0)-=xd*af; /* Add to the position change vector */
       MATRIX(dxdy, k, 0)+=xd*af;
       MATRIX(dxdy, j, 1)-=yd*af;
@@ -332,7 +340,9 @@ int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *r
 
 /**
  * \function igraph_layout_fruchterman_reingold_3d
- * \brief This is the 3D version of the force based
+ * \brief 3D Fruchterman-Reingold algorithm.
+ * 
+ * This is the 3D version of the force based
  * Fruchterman-Reingold layout (see \ref
  * igraph_layout_fruchterman_reingold for the 2D version
  *
@@ -352,6 +362,9 @@ int igraph_layout_fruchterman_reingold(const igraph_t *graph, igraph_matrix_t *r
  * \param use_seed Logical, if true the supplied values in the
  *        \p res argument are used as an initial layout, if
  *        false a random initial layout is used.
+ * \param weight Pointer to a vector containing edge weights, 
+ *        the attraction along the edges will be multiplied by these. 
+ *        It will be ignored if it is a null-pointer.
  * \return Error code.
  *
  * Added in version 0.2.</para><para>
@@ -367,7 +380,8 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 					  igraph_integer_t niter, igraph_real_t maxdelta,
 					  igraph_real_t volume, igraph_real_t coolexp,
 					  igraph_real_t repulserad,
-					  igraph_bool_t use_seed) {
+					  igraph_bool_t use_seed,
+					  const igraph_vector_t *weight) {
   
   igraph_real_t frk, t, ded, xd, yd, zd;
   igraph_matrix_t dxdydz;
@@ -377,6 +391,10 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
   long int no_of_nodes=igraph_vcount(graph);
   igraph_eit_t edgeit;
   igraph_integer_t from, to;
+
+  if (weight && igraph_vector_size(weight) != igraph_ecount(graph)) {
+    IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+  }
   
   IGRAPH_CHECK(igraph_matrix_init(&dxdydz, no_of_nodes, 3));
   IGRAPH_FINALLY(igraph_matrix_destroy, &dxdydz);
@@ -429,7 +447,9 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
     /*Calculate the attractive "force"*/
     IGRAPH_EIT_RESET(edgeit);
     while (!IGRAPH_EIT_END(edgeit)) {
-      igraph_edge(graph, IGRAPH_EIT_GET(edgeit), &from, &to);
+      long int edge=IGRAPH_EIT_GET(edgeit);
+      igraph_real_t w= weight ? VECTOR(*weight)[edge] : 1.0;
+      igraph_edge(graph, edge, &from, &to);
       j=from;
       k=to;
       xd=MATRIX(*res, j, 0)-MATRIX(*res, k, 0);
@@ -441,7 +461,7 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 	yd/=ded;
 	zd/=ded;
       }
-      af=ded*ded/frk;
+      af=ded*ded/frk*w;
       MATRIX(dxdydz, j, 0)-=xd*af;   /*Add to the position change vector*/
       MATRIX(dxdydz, k, 0)+=xd*af;
       MATRIX(dxdydz, j, 1)-=yd*af;
@@ -479,8 +499,7 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 /**
  * \ingroup layout
  * \function igraph_layout_kamada_kawai
- * \brief Places the vertices on a plane according the Kamada-Kawai
- * algorithm. 
+ * \brief Places the vertices on a plane according the Kamada-Kawai algorithm. 
  *
  * </para><para>
  * This is a force directed layout, see  Kamada, T. and Kawai, S.: An
@@ -496,6 +515,9 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
  * \param initemp Sets the initial temperature for the annealing.
  * \param coolexp The cooling exponent of the annealing.
  * \param kkconst The Kamada-Kawai vertex attraction constant.
+ * \param use_seed Boolean, whether to use the values cupplied in the \p res 
+ *     argument as the initial configuration. If zero then a random initial 
+ *     configuration is used.
  * \return Error code.
  * 
  * Time complexity: O(|V|^2) for each
@@ -506,7 +528,7 @@ int igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 int igraph_layout_kamada_kawai(const igraph_t *graph, igraph_matrix_t *res,
 			       igraph_integer_t niter, igraph_real_t sigma, 
 			       igraph_real_t initemp, igraph_real_t coolexp,
-			       igraph_real_t kkconst) {
+			       igraph_real_t kkconst, igraph_bool_t use_seed) {
 
   igraph_real_t temp, candx, candy;
   igraph_real_t dpot, odis, ndis, osqd, nsqd;
@@ -524,10 +546,12 @@ int igraph_layout_kamada_kawai(const igraph_t *graph, igraph_matrix_t *res,
   IGRAPH_MATRIX_INIT_FINALLY(&elen, n, n);
   IGRAPH_CHECK(igraph_shortest_paths(graph, &elen, igraph_vss_all(), 
 				     IGRAPH_ALL));
-  for (i=0; i<n; i++) {
-    MATRIX(elen, i, i) = sqrt(n);
-    MATRIX(*res, i, 0) = RNG_NORMAL(0, n/4.0);
-    MATRIX(*res, i, 1) = RNG_NORMAL(0, n/4.0);
+  if (!use_seed) {
+    for (i=0; i<n; i++) {
+      MATRIX(elen, i, i) = sqrt(n);
+      MATRIX(*res, i, 0) = RNG_NORMAL(0, n/4.0);
+      MATRIX(*res, i, 1) = RNG_NORMAL(0, n/4.0);
+    }
   }
   
   /*Perform the annealing loop*/
@@ -578,8 +602,9 @@ int igraph_layout_kamada_kawai(const igraph_t *graph, igraph_matrix_t *res,
 
 /**
  * \function igraph_layout_kamada_kawai_3d
- * \brief 3D version of the force based Kamada-Kawai layout, the pair
- * of the \ref igraph_layout_kamada_kawai 2D layout generator
+ * \brief 3D version of the force based Kamada-Kawai layout.
+ *
+ * The pair of the \ref igraph_layout_kamada_kawai 2D layout generator
  * 
  * </para><para>
  * This function was ported from the SNA R package.
@@ -592,6 +617,9 @@ int igraph_layout_kamada_kawai(const igraph_t *graph, igraph_matrix_t *res,
  * \param initemp Sets the initial temperature for the annealing.
  * \param coolexp The cooling exponent of the annealing.
  * \param kkconst The Kamada-Kawai vertex attraction constant.
+ * \param use_seed Boolean, whether to use the values cupplied in the \p res 
+ *     argument as the initial configuration. If zero then a random initial 
+ *     configuration is used.
  * \return Error code.
  * 
  * Added in version 0.2.</para><para>
@@ -604,7 +632,7 @@ int igraph_layout_kamada_kawai(const igraph_t *graph, igraph_matrix_t *res,
 int igraph_layout_kamada_kawai_3d(const igraph_t *graph, igraph_matrix_t *res,
 				  igraph_integer_t niter, igraph_real_t sigma, 
 				  igraph_real_t initemp, igraph_real_t coolexp, 
-				  igraph_real_t kkconst) {
+				  igraph_real_t kkconst, igraph_bool_t use_seed) {
   igraph_real_t temp, candx, candy, candz;
   igraph_real_t dpot, odis, ndis, osqd, nsqd;
   long int i,j,k;
@@ -618,6 +646,15 @@ int igraph_layout_kamada_kawai_3d(const igraph_t *graph, igraph_matrix_t *res,
   IGRAPH_CHECK(igraph_shortest_paths(graph, &elen, igraph_vss_all(), 
 				     IGRAPH_ALL));
   
+  if (!use_seed) {
+    for (i=0; i<no_of_nodes; i++) {
+      MATRIX(elen, i, i) = sqrt(no_of_nodes);
+      MATRIX(*res, i, 0) = RNG_NORMAL(0, no_of_nodes/4.0);
+      MATRIX(*res, i, 1) = RNG_NORMAL(0, no_of_nodes/4.0);
+      MATRIX(*res, i, 2) = RNG_NORMAL(0, no_of_nodes/4.0);
+    }
+  }
+
   temp=initemp;
   for(i=0;i<niter;i++){
     if (i%10 == 0)
@@ -779,7 +816,7 @@ int igraph_layout_lgl(const igraph_t *graph, igraph_matrix_t *res,
 
   /* Place the vertices randomly */
   IGRAPH_CHECK(igraph_layout_random(graph, res));
-  igraph_matrix_multiply(res, 1e6);
+  igraph_matrix_scale(res, 1e6);
   
   /* This is the grid for calculating the vertices near to a given vertex */  
   IGRAPH_CHECK(igraph_2dgrid_init(&grid, res, 
@@ -883,6 +920,10 @@ int igraph_layout_lgl(const igraph_t *graph, igraph_matrix_t *res,
       igraph_real_t t=maxdelta*pow((maxit-it)/(double)maxit, coolexp);
       long int vid, nei;
 
+	  IGRAPH_PROGRESS("Large graph layout",
+	    100.0*((actlayer-1.0)/(no_of_layers-1.0)+((float)it)/(maxit*(no_of_layers-1.0))),
+		0);
+
       /* init */
       igraph_vector_null(&forcex);
       igraph_vector_null(&forcey);
@@ -951,6 +992,7 @@ int igraph_layout_lgl(const igraph_t *graph, igraph_matrix_t *res,
     }
   }
 
+  IGRAPH_PROGRESS("Large graph layout", 100.0, 0);
   igraph_destroy(&mst);
   igraph_vector_destroy(&vids);
   igraph_vector_destroy(&layers);
@@ -1021,7 +1063,7 @@ int igraph_layout_grid_fruchterman_reingold(const igraph_t *graph,
   /* initial layout */
   if (!use_seed) {
     IGRAPH_CHECK(igraph_layout_random(graph, res));
-    igraph_matrix_multiply(res, sqrt(area/M_PI));
+    igraph_matrix_scale(res, sqrt(area/M_PI));
   }
   
   /* make grid */
@@ -1170,7 +1212,7 @@ int igraph_layout_reingold_tilford(const igraph_t *graph,
   long int no_of_nodes=igraph_vcount(graph);
   long int i, n, j;
   igraph_dqueue_t q=IGRAPH_DQUEUE_NULL;
-  igraph_i_adjlist_t allneis;
+  igraph_adjlist_t allneis;
   igraph_vector_t *neis;
   struct igraph_i_reingold_tilford_vertex *vdata;
   
@@ -1181,10 +1223,10 @@ int igraph_layout_reingold_tilford(const igraph_t *graph,
   IGRAPH_CHECK(igraph_matrix_resize(res, no_of_nodes, 2));
   IGRAPH_DQUEUE_INIT_FINALLY(&q, 100);
   
-  IGRAPH_CHECK(igraph_i_adjlist_init(graph, &allneis, IGRAPH_ALL));
-  IGRAPH_FINALLY(igraph_i_adjlist_destroy, &allneis);
+  IGRAPH_CHECK(igraph_adjlist_init(graph, &allneis, IGRAPH_ALL));
+  IGRAPH_FINALLY(igraph_adjlist_destroy, &allneis);
   
-  vdata=Calloc(no_of_nodes, struct igraph_i_reingold_tilford_vertex);
+  vdata=igraph_Calloc(no_of_nodes, struct igraph_i_reingold_tilford_vertex);
   if (vdata==0) {
     IGRAPH_ERROR("igraph_layout_reingold_tilford failed", IGRAPH_ENOMEM);
   }
@@ -1207,7 +1249,7 @@ int igraph_layout_reingold_tilford(const igraph_t *graph,
   while (!igraph_dqueue_empty(&q)) {
     long int actnode=igraph_dqueue_pop(&q);
     long int actdist=igraph_dqueue_pop(&q);
-    neis=igraph_i_adjlist_get(&allneis, actnode);
+    neis=igraph_adjlist_get(&allneis, actnode);
     n=igraph_vector_size(neis);
     for (j=0; j<n; j++) {
       long int neighbor=VECTOR(*neis)[j];
@@ -1228,7 +1270,7 @@ int igraph_layout_reingold_tilford(const igraph_t *graph,
   igraph_i_layout_reingold_tilford_calc_coords(vdata, res, root, no_of_nodes, vdata[root].offset);
   
   igraph_dqueue_destroy(&q);
-  igraph_i_adjlist_destroy(&allneis);
+  igraph_adjlist_destroy(&allneis);
   igraph_free(vdata);
   IGRAPH_FINALLY_CLEAN(3);
   
@@ -1401,6 +1443,368 @@ int igraph_layout_reingold_tilford_circular(const igraph_t *graph,
   return 0;
 }
 
+#define COULOMBS_CONSTANT 8987500000.0
+
+igraph_real_t igraph_i_distance_between(const igraph_matrix_t *c, long int a, 
+					long int b) {
+  igraph_real_t diffx=MATRIX(*c, a, 0)-MATRIX(*c, b, 0);
+  igraph_real_t diffy=MATRIX(*c, a, 1)-MATRIX(*c, b, 1);
+  return sqrt( diffx*diffx + diffy*diffy );
+} 
+
+int igraph_i_determine_electric_axal_forces(const igraph_matrix_t *pos,
+					    igraph_real_t *x,
+					    igraph_real_t *y,
+					    igraph_real_t directed_force,
+					    igraph_real_t distance,
+					    long int other_node,
+					    long int this_node) {
+  
+  // We know what the directed force is.  We now need to translate it
+  // into the appropriate x and y componenets.
+  // First, assume: 
+  //                 other_node
+  //                    /|
+  //  directed_force  /  |
+  //                /    | y
+  //              /______|
+  //    this_node     x         
+  //
+  // other_node.x > this_node.x
+  // other_node.y > this_node.y
+  // the force will be on this_node away from other_node
+  
+  // the proportion (distance/y_distance) is equal to the proportion
+  // (directed_force/y_force), as the two triangles are similar.
+  // therefore, the magnitude of y_force = (directed_force*y_distance)/distance
+  // the sign of y_force is negative, away from other_node
+  
+  igraph_real_t x_distance, y_distance;
+  y_distance = MATRIX(*pos, other_node, 1)-MATRIX(*pos, this_node, 1);
+  if (y_distance < 0) { y_distance = -y_distance; }
+  *y = -1 * ((directed_force * y_distance) / distance);
+
+  // the x component works in exactly the same way.
+  x_distance = MATRIX(*pos, other_node, 0)-MATRIX(*pos, this_node, 0);
+  if (x_distance < 0) { x_distance = -x_distance; }
+  *x = -1 * ((directed_force * x_distance) / distance);
+  
+   // Now we need to reverse the polarity of our answers based on the falsness
+   // of our assumptions.
+  if (MATRIX(*pos, other_node, 0) < MATRIX(*pos, this_node, 0)) {
+    *x = *x * -1;
+  }
+  if (MATRIX(*pos, other_node, 1) < MATRIX(*pos, this_node, 1)) {
+    *y = *y * -1;
+  }
+  
+  return 0;
+}
+  
+int igraph_i_apply_electrical_force(const igraph_matrix_t *pos,
+				    igraph_vector_t *pending_forces_x,
+				    igraph_vector_t *pending_forces_y,
+				    long int other_node, long int this_node,
+				    igraph_real_t node_charge, 
+				    igraph_real_t distance) {
+
+  igraph_real_t directed_force = COULOMBS_CONSTANT * 
+    ((node_charge * node_charge)/(distance * distance));
+  
+  igraph_real_t x_force, y_force;
+  igraph_i_determine_electric_axal_forces(pos, &x_force, &y_force, 
+					  directed_force, distance, 
+					  other_node, this_node);
+
+  VECTOR(*pending_forces_x)[this_node] += x_force;
+  VECTOR(*pending_forces_y)[this_node] += y_force;
+  VECTOR(*pending_forces_x)[other_node] -= x_force;
+  VECTOR(*pending_forces_y)[other_node] -= y_force;
+
+  return 0;
+}
+
+int igraph_i_determine_spring_axal_forces(const igraph_matrix_t *pos,
+					  igraph_real_t *x, igraph_real_t *y,
+					  igraph_real_t directed_force,
+					  igraph_real_t distance,
+					  int spring_length,
+					  long int other_node, long int this_node) {
+
+  // if the spring is just the right size, the forces will be 0, so we can
+  // skip the computation.
+  //
+  // if the spring is too long, our forces will be identical to those computed
+  // by determine_electrical_axal_forces() (this_node will be pulled toward
+  // other_node).
+  //
+  // if the spring is too short, our forces will be the opposite of those
+  // computed by determine_electrical_axal_forces() (this_node will be pushed
+  // away from other_node)
+  //
+  // finally, since both nodes are movable, only one-half of the total force
+  // should be applied to each node, so half the forces for our answer.
+  
+  if (distance == spring_length) {
+    *x = 0.0;
+    *y = 0.0;
+  } else {
+    igraph_i_determine_electric_axal_forces(pos, x, y, directed_force, distance, 
+					    other_node, this_node);
+    if (distance < spring_length) {
+      *x = -1 * *x;
+      *y = -1 * *y;
+    }
+    *x = 0.5 * *x;
+    *y = 0.5 * *y;
+  }
+  
+  return 0;
+}
+
+int igraph_i_apply_spring_force(const igraph_matrix_t *pos, 
+				igraph_vector_t *pending_forces_x,
+				igraph_vector_t *pending_forces_y,
+				long int other_node,
+				long int this_node, int spring_length,
+				igraph_real_t spring_constant) {
+
+  // determined using Hooke's Law:
+  //   force = -kx
+  // where:
+  //   k = spring constant
+  //   x = displacement from ideal length in meters
+  
+  igraph_real_t distance, displacement, directed_force, x_force, y_force;
+  distance = igraph_i_distance_between(pos, other_node, this_node);
+  // let's protect ourselves from division by zero by ignoring two nodes that
+  // happen to be in the same place.  Since we separate all nodes before we
+  // work on any of them, this will only happen in extremely rare circumstances,
+  // and when it does, electrical force will probably push one or both of them
+   // one way or another anyway.
+  if (distance == 0.0) {
+    return 0;
+  }
+  
+  displacement = distance - spring_length;
+  if (displacement < 0) {
+    displacement = -displacement;
+  }
+  directed_force = -1 * spring_constant * displacement;
+  // remember, this is force directed away from the spring;
+  // a negative number is back towards the spring (or, in our case, back towards
+   // the other node)
+  
+  // get the force that should be applied to >this< node
+  igraph_i_determine_spring_axal_forces(pos, &x_force, &y_force, 
+					directed_force, distance, spring_length,
+					other_node, this_node);
+  
+  VECTOR(*pending_forces_x)[this_node] += x_force;
+  VECTOR(*pending_forces_y)[this_node] += y_force;
+  VECTOR(*pending_forces_x)[other_node] -= x_force;
+  VECTOR(*pending_forces_y)[other_node] -= y_force;
+
+  return 0;
+}
+
+int igraph_i_move_nodes(igraph_matrix_t *pos, 
+			const igraph_vector_t *pending_forces_x, 
+			const igraph_vector_t *pending_forces_y,
+			igraph_real_t node_mass,
+			igraph_real_t max_sa_movement) {
+  
+  // Since each iteration is isolated, time is constant at 1.
+  // Therefore:
+  //   Force effects acceleration.
+  //   acceleration (d(velocity)/time) = velocity
+  //   velocity (d(displacement)/time) = displacement
+  //   displacement = acceleration
+  
+  // determined using Newton's second law:
+  //   sum(F) = ma
+  // therefore:
+  //   acceleration = force / mass
+  //   velocity     = force / mass
+  //   displacement = force / mass
+
+  long int this_node, no_of_nodes=igraph_vector_size(pending_forces_x);
+
+  for (this_node=0; this_node < no_of_nodes; this_node++) {
+
+    igraph_real_t x_movement, y_movement;
+
+    x_movement = VECTOR(*pending_forces_x)[this_node] / node_mass;
+    if (x_movement > max_sa_movement) {
+      x_movement = max_sa_movement;
+    } else if (x_movement < -max_sa_movement) {
+      x_movement = -max_sa_movement;
+    }
+
+    y_movement = VECTOR(*pending_forces_y)[this_node] / node_mass;
+    if (y_movement > max_sa_movement) {
+      y_movement = max_sa_movement;
+    } else if (y_movement < -max_sa_movement) {
+      y_movement = -max_sa_movement;
+    }
+
+    MATRIX(*pos, this_node, 0) += x_movement;
+    MATRIX(*pos, this_node, 1) += y_movement;
+
+  }
+  return 0;
+}
+
+/**
+ * \function igraph_layout_graphopt
+ * \brief Optimizes vertex layout via the graphopt algorithm.
+ * 
+ * </para><para>
+ * This is a port of the graphopt layout algorithm by Michael Schmuhl.
+ * graphopt version 0.4.1 was rewritten in C and the support for 
+ * layers was removed (might be added later) and a code was a bit 
+ * reorganized to avoid some unneccessary steps is the node charge (see below) 
+ * is zero.
+ * 
+ * </para><para>
+ * graphopt uses physical analogies for defining attracting and repelling 
+ * forces among the vertices and then the physical system is simulated 
+ * until it reaches an equilibrium. (There is no simulated annealing or 
+ * anything like that, so a stable fixed point is not guaranteed.)
+ * 
+ * </para><para>
+ * See also http://www.schmuhl.org/graphopt/ for the original graphopt.
+ * \param graph The input graph.
+ * \param res Pointer to an initialized matrix, the result will be stored here
+ *    and its initial contents is used the starting point of the simulation
+ *    if the \p use_seed argument is true. Note that in this case the 
+ *    matrix should have the proper size, otherwise a warning is issued and 
+ *    the supplied values are ignored. If no starting positions are given 
+ *    (or they are invalid) then a random staring position is used. 
+ *    The matrix will be resized if needed.
+ * \param niter Integer constant, the number of iterations to perform.
+ *    Should be a couple of hundred in general. If you have a large graph 
+ *    then you might want to only do a few iterations and then check the 
+ *    result. If it is not good enough you can feed it in again in 
+ *    the \p res argument. The original graphopt default if 500.
+ * \param node_charge The charge of the vertices, used to calculate electric
+ *    repulsion. The original graphopt default is 0.001.
+ * \param node_mass The mass of the vertices, used for the spring forces.
+ *    The original graphopt defaults to 30.
+ * \param spring_length The length of the springs, an integer number.
+ *    The original graphopt defaults to zero.
+ * \param spring_constant The spring constant, the original graphopt defaults 
+ *    to one.
+ * \param max_sa_movement Real constant, it gives the maximum amount of movement 
+ *    allowed in a single step along a single axis. The original graphopt 
+ *    default is 5.
+ * \param use_seed Logical scalar, whether to use the positions in \p res as
+ *    a starting configuration. See also \p res above.
+ * \return Error code.
+ * 
+ * Time complexity: O(n (|V|^2+|E|) ), n is the number of iterations, 
+ * |V| is the number of vertices, |E| the number
+ * of edges. If \p node_charge is zero then it is only O(n|E|).
+ */
+
+int igraph_layout_graphopt(const igraph_t *graph, igraph_matrix_t *res, 
+			   igraph_integer_t niter,
+			   igraph_real_t node_charge, igraph_real_t node_mass,
+			   igraph_integer_t spring_length,
+			   igraph_real_t spring_constant, 
+			   igraph_real_t max_sa_movement,
+			   igraph_bool_t use_seed) {
+
+  long int no_of_nodes=igraph_vcount(graph);
+  long int no_of_edges=igraph_ecount(graph);
+  int my_spring_length=spring_length;
+  igraph_vector_t pending_forces_x, pending_forces_y;
+  /* Set a flag to calculate (or not) the electrical forces that the nodes */
+  /* apply on each other based on if both node types' charges are zero. */
+  igraph_bool_t apply_electric_charges= (node_charge!=0);
+  
+  long int this_node, other_node, edge;
+  igraph_real_t distance;
+  long int i;
+
+  IGRAPH_VECTOR_INIT_FINALLY(&pending_forces_x, no_of_nodes);
+  IGRAPH_VECTOR_INIT_FINALLY(&pending_forces_y, no_of_nodes);
+  
+  if (use_seed) {
+    if (igraph_matrix_nrow(res) != no_of_nodes ||
+	igraph_matrix_ncol(res) != 2) {
+      IGRAPH_WARNING("Invalid size for initial matrix, starting from random layout");
+      IGRAPH_CHECK(igraph_layout_random(graph, res));
+    }
+  } else {
+    IGRAPH_CHECK(igraph_layout_random(graph, res));
+  }
+
+  igraph_progress("Graphopt layout", 0, NULL);
+  for(i=niter;i>0;i--) {
+    /* Report progress in approx. every 100th step */
+    if (i%10 == 0) {
+      igraph_progress("Graphopt layout", 100.0-100.0*i/niter, NULL);
+    }
+    
+    /* Clear pending forces on all nodes */
+    igraph_vector_null(&pending_forces_x);
+    igraph_vector_null(&pending_forces_y);
+    
+    // Apply electrical force applied by all other nodes
+    if (apply_electric_charges) {
+      // Iterate through all nodes
+      for (this_node = 0; this_node < no_of_nodes; this_node++) {
+	IGRAPH_ALLOW_INTERRUPTION();
+	for (other_node = this_node + 1; 
+	     other_node < no_of_nodes; 
+	     other_node++) {
+	  distance = igraph_i_distance_between(res, this_node, other_node);
+	  // let's protect ourselves from division by zero by ignoring
+	  // two nodes that happen to be in the same place.  Since we 
+	  // separate all nodes before we work on any of them, this 
+	  // will only happen in extremely rare circumstances, and when
+	  // it does, springs will probably pull them apart anyway.
+	  // also, if we are more than 50 away, the electric force 
+	  // will be negligable.  
+	  // ***** may not always be desirable ****
+	  if ((distance != 0.0) && (distance < 500.0)) {
+	    //	  if (distance != 0.0) {
+	    // Apply electrical force from node(counter2) on 
+	    // node(counter)
+	    igraph_i_apply_electrical_force(res, &pending_forces_x, 
+					    &pending_forces_y, 
+					    other_node, this_node, 
+					    node_charge,
+					    distance);
+	  }
+	}
+      }
+    }
+      
+    // Apply force from springs
+    for (edge = 0; edge < no_of_edges; edge++) {
+      long int this_node=IGRAPH_FROM(graph, edge);
+      long int other_node=IGRAPH_TO(graph, edge);
+      // Apply spring force on both nodes
+      igraph_i_apply_spring_force(res, &pending_forces_x, &pending_forces_y,
+				  other_node, this_node, my_spring_length, 
+				  spring_constant);
+    }
+  
+    // Effect the movement of the nodes based on all pending forces
+    igraph_i_move_nodes(res, &pending_forces_x, &pending_forces_y, node_mass,
+			max_sa_movement);
+  }
+  igraph_progress("Graphopt layout", 100, NULL);
+
+  igraph_vector_destroy(&pending_forces_y);
+  igraph_vector_destroy(&pending_forces_x);
+  IGRAPH_FINALLY_CLEAN(2);
+  
+  return 0;
+}
+
 int igraph_i_layout_merge_dla(igraph_i_layout_mergegrid_t *grid, 
 			      long int actg, igraph_real_t *x, igraph_real_t *y, igraph_real_t r,
 			      igraph_real_t cx, igraph_real_t cy, igraph_real_t startr, 
